package org.rcisoft.bus.util;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.annotation.Excel;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.DVConstraint;
import org.apache.poi.hssf.usermodel.HSSFDataValidation;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.usermodel.XSSFDataValidation;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.sys.wbac.user.dto.ExcelDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


public class ExportExcelUtil {
    private static final Logger log = LoggerFactory.getLogger(ExportExcelUtil.class);

    public ExportExcelUtil() {
    }



    /**
     * 为Excel表格添加下拉选项
     * 解决单元格在某个时间点只能访问有限数据量而导致无法显示下拉选项的问题
     * 适用于下拉选项总字节数大于 255 个字节的情况
     * 缺点：会创建一个隐藏的sheet用于存放下拉选项
     *
     * @param formulaString 下拉选项数据
     * @param workbook      Excel工作簿
     * @param sheet         设置下拉选项的工作表
     * @param firstRow      起始行
     * @param lastRow       终止行
     * @param firstCol      起始列
     * @param lastCol       终止列
     */
    public static void XSSFSetDropDownAndHidden(String[] formulaString, Workbook workbook, Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) {
        // 计算下拉选项全部字节数
        int totalBytes = Arrays.stream(formulaString)
                .mapToInt(str -> str.getBytes(StandardCharsets.UTF_8).length)
                .sum();
        // 创建一个数据验证助手
        DataValidationHelper dvHelper = sheet.getDataValidationHelper();
        if (totalBytes > 255) {
            // 当总选项字节数大于 255 时创建隐藏工作表用于存放下拉选项
            // 创建sheet，写入枚举项
            //判断当前sheet不存在则创建
            Sheet hideSheet  = workbook.getSheet("hiddenSheet");
            if (workbook.getSheet("hiddenSheet") == null) {
                hideSheet = workbook.createSheet("hiddenSheet");
            }

            for (int i = 0; i < formulaString.length; i++) {
                // 将下拉选项值存放在目标工作表下拉项目相同列
                hideSheet.createRow(i).createCell(firstCol).setCellValue(formulaString[i]);
            }
            // 创建名称，可被其他单元格引用
            //判断当前名称不存在则创建
            Name hiddenSheetName = workbook.getName("hidden");
            if (workbook.getName("hidden") == null) {
                hiddenSheetName = workbook.createName();
                hiddenSheetName.setNameName("hidden");
            }
            // 设置名称引用的公式
            // 使用像'A1：B1'这样的相对值会导致在Microsoft Excel中使用工作簿时名称所指向的单元格的意外移动，
            // 通常使用绝对引用，例如'$A$1:$B$1'可以避免这种情况。
            // 根据 firstCol 计算引用索引字母
            StringBuilder columnLetter = new StringBuilder();
            int columnIndex = firstCol;
            while (columnIndex >= 0) {
                int rem = columnIndex % 26;
                columnLetter.append((char) ('A' + rem));
                columnIndex = (columnIndex / 26) - 1;
            }
            columnLetter.reverse();
            hiddenSheetName.setRefersToFormula("hiddenSheet!$" + columnLetter + "$1:$" + columnLetter + "$" + formulaString.length);
            // 获取上文名称内数据
            DataValidationConstraint constraint = dvHelper.createFormulaListConstraint("hidden");
            // 设置下拉框位置
            CellRangeAddressList addressList = new CellRangeAddressList(firstRow, lastRow, firstCol, lastCol);
            DataValidation dataValidation = dvHelper.createValidation(constraint, addressList);
            // 处理Excel兼容性问题
            if (dataValidation instanceof XSSFDataValidation) {
                // 数据校验
                dataValidation.setSuppressDropDownArrow(true);
                dataValidation.setShowErrorBox(true);
            } else {
                dataValidation.setSuppressDropDownArrow(false);
            }
            // 作用在目标sheet上
            sheet.addValidationData(dataValidation);
            // 设置hiddenSheet隐藏
            int hideSheetIndex = workbook.getSheetIndex(hideSheet);
            workbook.setSheetHidden(hideSheetIndex, true);
        } else {
            // 当总选项字节数不大于 255 时使用传统方式创建下拉选项
            // 创建下拉列表约束
            DataValidationConstraint dvConstraint = dvHelper.createExplicitListConstraint(formulaString);
            // 设置数据验证的单元格区域（从firstRow行firstCol列到lastRow行lastCol列）
            CellRangeAddressList addressList = new CellRangeAddressList(firstRow, lastRow, firstCol, lastCol);
            // 创建数据验证对象并添加到工作表
            DataValidation validation = dvHelper.createValidation(dvConstraint, addressList);
            sheet.addValidationData(validation);
        }
    }

    // 调用方法
    public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, HttpServletResponse response, String[] options) {
        ExportParams exportParams = new ExportParams(title, sheetName);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }

        defaultExport(list, pojoClass, fileName, response, exportParams, options);
    }

    private static void defaultExport(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, ExportParams exportParams, String[] options) {
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
        if (workbook != null) {
            setDropDownListAndWriteWorkbook(workbook, options, response, fileName);
        }
    }

    public static void setDropDownListAndWriteWorkbook(Workbook workbook, String[] options, HttpServletResponse response, String fileName) {
        // 设置下拉框的数据来源
        Sheet sheet = workbook.getSheetAt(0);
        DataValidationHelper dvHelper = sheet.getDataValidationHelper();
        // 创建一个数据验证约束，这里设置数据长度 1 - 100
        DataValidationConstraint dvConstraint = dvHelper.createTextLengthConstraint(DataValidationConstraint.OperatorType.BETWEEN, "1", "100");
        // 设置数据验证的单元格区域（第 1 行第 1 列）
        CellRangeAddressList addressList = new CellRangeAddressList(0, 0, 0, 0);
        // 创建数据验证对象
        DataValidation validation = dvHelper.createValidation(dvConstraint, addressList);
        // 创建下拉列表约束
        dvConstraint = dvHelper.createExplicitListConstraint(options);
        // 单位名称列 第一列
        addressList = new CellRangeAddressList(1, 100000, 0, 0);
        validation = dvHelper.createValidation(dvConstraint, addressList);
        // 将数据验证添加到工作表
        sheet.addValidationData(validation);

        // 写出工作簿到响应流中
        ServletOutputStream outputStream = null;
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
        } catch (IOException e) {
            throw new CyServiceException(e.getMessage());
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException e) {
                log.info("context:" + e);
            }
        }
    }
    // 设置工作簿的下拉框
    public static void setDropDownList(Workbook workbook,String[] options) {

        // 设置下拉框的数据来源
        Sheet sheet = workbook.getSheetAt(0);
        DataValidationHelper validationHelper = sheet.getDataValidationHelper();
        DataValidationConstraint constraint = validationHelper.createExplicitListConstraint(options);
        CellRangeAddressList addressList = new CellRangeAddressList(5, 5, 2, 2);
        DataValidation dataValidation = validationHelper.createValidation(constraint, addressList);

        // 将下拉框应用到指定单元格
        sheet.addValidationData(dataValidation);
    }

    public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, boolean isCreateHeader, HttpServletResponse response) {
        ExportParams exportParams = new ExportParams(title, sheetName);
        exportParams.setCreateHeadRows(isCreateHeader);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }

        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    public static void exportExcel(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response) {
        ExportParams exportParams = new ExportParams();
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }

        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    public static void exportExcel(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, String sheetName) {
        ExportParams exportParams = new ExportParams();
        exportParams.setSheetName(sheetName);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }

        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, HttpServletResponse response) {
        ExportParams exportParams = new ExportParams(title, sheetName);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }

        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    public static void exportExcel(List<Map<String, Object>> list, String fileName, HttpServletResponse response) {
        defaultExport(list, fileName, response);
    }

    private static void defaultExport(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, ExportParams exportParams) {
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
        if (workbook != null) {
            downLoadExcel(fileName, response, workbook);
        }

    }

    private static void defaultExport(List<Map<String, Object>> list, String fileName, HttpServletResponse response) {
        Workbook workbook = ExcelExportUtil.exportExcel(list, ExcelType.HSSF);
        if (workbook != null) {
            downLoadExcel(fileName, response, workbook);
        }

    }

    private static void downLoadExcel(String fileName, HttpServletResponse response, Workbook workbook) {
        ServletOutputStream outputStream = null;

        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
        } catch (IOException var12) {
            throw new CyServiceException(var12.getMessage());
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException var11) {
                log.info("context:" + var11);
            }

        }

    }

    public static <T> List<T> importExcel(String filePath, Integer titleRows, Integer headerRows, Class<T> pojoClass) {
        if (StringUtils.isBlank(filePath)) {
            return null;
        } else {
            ImportParams params = new ImportParams();
            params.setTitleRows(titleRows);
            params.setHeadRows(headerRows);
            List list = null;

            try {
                list = ExcelImportUtil.importExcel(new File(filePath), pojoClass, params);
                return list;
            } catch (NoSuchElementException var7) {
                throw new CyServiceException("模板不能为空");
            } catch (Exception var8) {
                log.info("context:" + var8);
                throw new CyServiceException(var8.getMessage());
            }
        }
    }

    public static <T> List<T> importExcel(MultipartFile file, Integer titleRows, Integer headerRows, Class<T> pojoClass) {
        if (file == null) {
            return null;
        } else {
            ImportParams params = new ImportParams();
            params.setTitleRows(titleRows);
            params.setHeadRows(headerRows);
            List list = null;

            try {
                list = ExcelImportUtil.importExcel(file.getInputStream(), pojoClass, params);
                return list;
            } catch (Exception var7) {
                throw new CyServiceException(502, "模板格式错误");
            }
        }
    }

    public static void exportExcelSelect(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, HttpServletResponse response, ExcelDTO excelDto) {
        Date date = new Date();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy年MM月dd日");
        String time = dateFormat.format(date);
        defaultExportSelect(list, pojoClass, fileName, response, new ExportParams(title, "填报日期:" + time, sheetName), excelDto);
    }

    private static void defaultExportSelect(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, ExportParams exportParams, ExcelDTO excelDto) {
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
        CellRangeAddress c = CellRangeAddress.valueOf("G3:H3");
        workbook.getSheetAt(0).setAutoFilter(c);
        if (null != excelDto.getPost() && excelDto.getPost().length > 0) {
            selectList(workbook, 3, 510, 7, 7, excelDto.getPost());
        }

        if (null != excelDto.getSex() && excelDto.getSex().length > 0) {
            selectList(workbook, 3, 510, 6, 6, excelDto.getSex());
        }

        if (workbook != null) {
            downLoadExcel(fileName, response, workbook);
        }

    }

    public static void selectList(Workbook workbook, int firstRow, int lastRow, int firstCol, int lastCol, String[] strings) {
        Sheet sheet = workbook.getSheetAt(0);
        CellRangeAddressList cellRangeAddressList = new CellRangeAddressList(firstRow, lastRow, firstCol, lastCol);
        DVConstraint dvConstraint = DVConstraint.createExplicitListConstraint(strings);
        HSSFDataValidation dataValidation = new HSSFDataValidation(cellRangeAddressList, dvConstraint);
        sheet.addValidationData(dataValidation);
    }

    public static boolean isFileInUser(MultipartFile file, Object o, int titleRow) throws Exception {
        HSSFWorkbook workbook = new HSSFWorkbook(file.getInputStream());
        HSSFSheet hssfSheet = workbook.getSheetAt(0);
        Row row = hssfSheet.getRow(titleRow);
        int cellNum = row.getLastCellNum();
        List<String> containsUtils = new ArrayList();

        for(int i = 0; i < cellNum; ++i) {
            containsUtils.add(row.getCell(i).getStringCellValue());
        }

        Class clazz = o.getClass();
        Field[] field = clazz.getDeclaredFields();
        List<String> fileArray = new ArrayList();
        Field[] var11 = field;
        int var12 = field.length;

        for(int var13 = 0; var13 < var12; ++var13) {
            Field field1 = var11[var13];
            field1.setAccessible(true);
            if (field1.isAnnotationPresent(Excel.class)) {
                Excel excel = (Excel)field1.getAnnotation(Excel.class);
                String name = excel.name();
                fileArray.add(name);
            }
        }

        return ((String)containsUtils.stream().sorted().collect(Collectors.joining())).equals(fileArray.stream().sorted().collect(Collectors.joining()));
    }
}
